@@ -77,6 +77,16 @@ sub match {
           . "/");
 
     my @values = $path =~ $self->{_compiled_regexp};
+
+    # the regex comments are how we know if we captured
+    # a splat or a megasplat
+    if( my @splat_or_megasplat
+            = $self->{_compiled_regexp} =~ /\(\?#((?:mega)?splat)\)/g ) {
+        for ( @values ) {
+            $_ = [ split '/' => $_ ] if ( shift @splat_or_megasplat ) =~ /megasplat/;
+        }
+    }
+
     Dancer::Logger::core("  --> got ".
         map { defined $_ ? $_ : 'undef' } @values) 
         if @values;
@@ -158,13 +168,14 @@ sub run {
     if ( $response && $response->is_forwarded ) {
         my $new_req = 
             Dancer::Request->forward($request, $response->{forward});
-
         my $marshalled = Dancer::Handler->handle_request($new_req);
 
         return Dancer::Response->new(
+            encoded => 1,
             status  => $marshalled->[0],
             headers => $marshalled->[1],
-            content => @{ $marshalled->[2] },
+            # if the forward failed with 404, marshalled->[2] is not an array, but a GLOB
+            content => ref($marshalled->[2]) eq "ARRAY" ? @{ $marshalled->[2] } : $marshalled->[2]
         );
     }
 
@@ -260,7 +271,6 @@ sub _init_prefix {
     }
     else {
         $self->{pattern} = $prefix . $self->pattern;
-        $self->{pattern} =~ s/\/$//;
     }
 
     return $prefix;
@@ -286,6 +296,7 @@ sub _build_regexp {
 
     if ($self->is_regexp) {
         $self->{_compiled_regexp} = $self->regexp || $self->pattern;
+        $self->{_compiled_regexp} = qr/^$self->{_compiled_regexp}$/;
         $self->{_should_capture} = 1;
     }
     else {
@@ -309,11 +320,13 @@ sub _build_regexp_from_string {
         }
     }
 
+    # parse megasplat
+    # we use {0,} instead of '*' not to fall in the splat rule
+    # same logic for [^\n] instead of '.'
+    $capture = 1 if $pattern =~ s!\Q**\E!(?#megasplat)([^\n]+)!g;
+
     # parse wildcards
-    if ($pattern =~ /\*/) {
-        $pattern =~ s/\*/\(\[\^\/\]\+\)/g;
-        $capture = 1;
-    }
+    $capture = 1 if $pattern =~ s!\*!(?#splat)([^/]+)!g;
 
     # escape dots
     $pattern =~ s/\./\\\./g if $pattern =~ /\./;
